/*
   Battery related stuff. ADC initialization. Low-Battery indicator API.
   
   Changelog:
   	2017-12-29: Initial release.

*/

#include <stdint.h>

#include <libopencm3/cm3/nvic.h>
#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/adc.h>

#include "../settings/battery.h"


volatile int_fast8_t lowBatt = 0;

void battery_init(){
	
	
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_ADC1);
	
	gpio_mode_setup(GPIOA, GPIO_MODE_ANALOG, GPIO_PUPD_NONE, GPIO0);
	nvic_enable_irq(NVIC_ADC_COMP_IRQ);
	
	/* 
	   Configure ADC: 
	   	
	   	- 239.5 cycles sampling time
	   	- Channel 0 (PA0) and Channel 17 (V_ref)
	   	- EndOfConversion interrupt
	   	- Enable V_ref
	*/
	
	ADC1_SMPR |= ADC_SMPR_SMP_239DOT5;
	ADC1_CHSELR |= ADC_CHSELR_CHSEL(0) | ADC_CHSELR_CHSEL(17);
	ADC1_IER |= ADC_IER_EOCIE;
	ADC_CCR(ADC1) |= ADC_CCR_VREFEN;
	
	/* Calibrate */
	
	ADC1_CR |= ADC_CR_ADCAL;
	while(ADC1_CR & ADC_CR_ADCAL){
	}
	
	/* Enable */
	
	ADC1_CR |= ADC_CR_ADEN;
	while(!ADC1_ISR & ADC_ISR_ADRDY){
	}
	ADC1_ISR |= ADC_ISR_ADRDY;
	
	/* Start */
	
	ADC1_CR |= ADC_CR_ADSTART;
}

void adc_comp_isr(){
	
	static int_fast8_t channel = 1;
	static int_fast16_t adcBattery = 4095;
	static int_fast16_t lowBattCount = 0;
	int_fast16_t adc;
	
	if(ADC1_ISR & ADC_ISR_EOC){
		
		adc = ADC1_DR;
		
		if(channel){
			
			/* Channel 0 (PA0) */
			
			adcBattery = adc;
			channel = 0;
		}
		else{
			
			/* Channel 17 (V_ref) */
			
			if(1024 * (int_fast32_t) adcBattery < THRESH_LOWER * (int_fast32_t) adc){
				
				++lowBattCount;
			}
			if(1024 * (int_fast32_t) adcBattery > THRESH_UPPER * (int_fast32_t) adc){
				
				lowBattCount = 0;
			}
			
			if(lowBattCount > THRESH_COUNT){
				
				lowBatt = 1;
			}
			
			channel = 1;
			ADC1_CR |= ADC_CR_ADSTART;
		}
	}	
}
