/*
   LED related stuff. PWM initialization. API.
   
   Changelog:
   	2017-12-29: Initial release.
   	
*/

#include <stdint.h>

#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/timer.h>

#include "led.h"


extern uint_fast8_t ledDcMul[4][3];
extern uint_fast8_t ledDcDiv[4][3];
extern int_fast16_t ledDcAdd[4][3];


void led_init(){
/*
	Setting up timer/gpio configuration for hardware PWM.
*/

	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_GPIOB);
	
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO6);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO7);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO8);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO9);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO10);
	gpio_set_output_options(GPIOA, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO11);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO0);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO1);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO4);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO5);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO14);
	gpio_set_output_options(GPIOB, GPIO_OTYPE_PP, GPIO_OSPEED_LOW, GPIO15);
	
	
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO6);
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO7);
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO8);
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO9);
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO10);
	gpio_mode_setup(GPIOA, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO11);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO0);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO1);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO4);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO5);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO14);
	gpio_mode_setup(GPIOB, GPIO_MODE_AF, GPIO_PUPD_NONE, GPIO15);
	
	
	gpio_set_af(GPIOA, GPIO_AF5, GPIO6);
	gpio_set_af(GPIOA, GPIO_AF5, GPIO7);
	gpio_set_af(GPIOA, GPIO_AF2, GPIO8);
	gpio_set_af(GPIOA, GPIO_AF2, GPIO9);
	gpio_set_af(GPIOA, GPIO_AF2, GPIO10);
	gpio_set_af(GPIOA, GPIO_AF2, GPIO11);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO0);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO1);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO4);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO5);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO14);
	gpio_set_af(GPIOB, GPIO_AF1, GPIO15);
	
	rcc_periph_clock_enable(RCC_TIM1);
	rcc_periph_clock_enable(RCC_TIM3);
	rcc_periph_clock_enable(RCC_TIM15);
	rcc_periph_clock_enable(RCC_TIM16);
	rcc_periph_clock_enable(RCC_TIM17);
	
	
	TIM1_ARR = PWM_MAX;
	TIM1_CCR1 = 0;
	TIM1_CCR2 = 0;
	TIM1_CCR3 = 0;
	TIM1_CCR4 = 0;
	
	TIM3_ARR = PWM_MAX;
	TIM3_CCR1 = 0;
	TIM3_CCR2 = 0;
	TIM3_CCR3 = 0;
	TIM3_CCR4 = 0;
	
	TIM15_ARR = PWM_MAX;
	TIM15_CCR1 = 0;
	TIM15_CCR2 = 0;
	
	TIM16_ARR = PWM_MAX;
	TIM16_CCR1 = 0;
	
	TIM17_ARR = PWM_MAX;
	TIM17_CCR1 = 0;
	
	
	TIM1_CCER |= TIM_CCER_CC1E | TIM_CCER_CC2E | TIM_CCER_CC3E | TIM_CCER_CC4E;
	TIM1_CCMR1 |= TIM_CCMR1_OC1M_PWM1 | TIM_CCMR1_OC1PE | TIM_CCMR1_OC2M_PWM1 | TIM_CCMR1_OC2PE;
	TIM1_CCMR2 |= TIM_CCMR2_OC3M_PWM1 | TIM_CCMR2_OC3PE | TIM_CCMR2_OC4M_PWM1 | TIM_CCMR2_OC4PE;
	TIM1_BDTR |= TIM_BDTR_MOE;
	TIM1_EGR |= TIM_EGR_UG;
	
	TIM3_CCER |= TIM_CCER_CC1E | TIM_CCER_CC2E | TIM_CCER_CC3E | TIM_CCER_CC4E;
	TIM3_CCMR1 |= TIM_CCMR1_OC1M_PWM1 | TIM_CCMR1_OC1PE | TIM_CCMR1_OC2M_PWM1 | TIM_CCMR1_OC2PE;
	TIM3_CCMR2 |= TIM_CCMR2_OC3M_PWM1 | TIM_CCMR2_OC3PE | TIM_CCMR2_OC4M_PWM1 | TIM_CCMR2_OC4PE;
	TIM3_EGR |= TIM_EGR_UG;
	
	TIM15_CCER |= TIM_CCER_CC1E | TIM_CCER_CC2E;
	TIM15_CCMR1 |= TIM_CCMR1_OC1M_PWM1 | TIM_CCMR1_OC1PE | TIM_CCMR1_OC2M_PWM1 | TIM_CCMR1_OC2PE;
	TIM15_BDTR |= TIM_BDTR_MOE;
	TIM15_EGR |= TIM_EGR_UG;
	
	TIM16_CCER |= TIM_CCER_CC1E;
	TIM16_CCMR1 |= TIM_CCMR1_OC1M_PWM1 | TIM_CCMR1_OC1PE;
	TIM16_BDTR |= TIM_BDTR_MOE;
	TIM16_EGR |= TIM_EGR_UG;
	
	TIM17_CCER |= TIM_CCER_CC1E;
	TIM17_CCMR1 |= TIM_CCMR1_OC1M_PWM1 | TIM_CCMR1_OC1PE;
	TIM17_BDTR |= TIM_BDTR_MOE;
	TIM17_EGR |= TIM_EGR_UG;
	
	
	TIM1_CR1 |= TIM_CR1_CEN;
	TIM3_CR1 |= TIM_CR1_CEN;
	TIM15_CR1 |= TIM_CR1_CEN;
	TIM16_CR1 |= TIM_CR1_CEN;
	TIM17_CR1 |= TIM_CR1_CEN;
}

int_fast8_t led_set(enum led l, enum color c, int_fast16_t v){
/*
	Setting color <c> of LED <l> to a PWM duty cycle of <v>/PWM_MAX.
*/
	
	if(l < 0 || l > 3 || c < 0 || c > 2){
		
		return 1;
	}
	
	v += ledDcAdd[l][c];
	v *= ledDcMul[l][c];
	v /= ledDcDiv[l][c];
	
	if(v < 0){
		
		v = 0;
	}
	
	if(v > PWM_MAX){
		
		v = PWM_MAX;
	}
	
	switch(l){
		case DOWN:
			switch(c){
				
				case RED:
					TIM3_CCR3 = v; break;
				case GREEN:
					TIM17_CCR1 = v; break;
				case BLUE:
					TIM16_CCR1 = v; break;
				default:
					return 1;
			} break;
			
		case RIGHT:
			switch(c){
				
				case RED:
					TIM15_CCR2 = v; break;
				case GREEN:
					TIM15_CCR1 = v; break;
				case BLUE:
					TIM3_CCR4 = v; break;
				default:
					return 1;
			} break;
			
		case UP:
			switch(c){
				
				case RED:
					TIM1_CCR3 = v; break;
				case GREEN:
					TIM1_CCR2 = v; break;
				case BLUE:
					TIM1_CCR1 = v; break;
				default:
					return 1;
			} break;
			
		case LEFT:
			switch(c){
				
				case RED:
					TIM3_CCR2 = v; break;
				case GREEN:
					TIM3_CCR1 = v; break;
				case BLUE:
					TIM1_CCR4 = v; break;
				default:
					return 1;
			} break;
		
		default:
			return 1;
	}
	
	return 0;
}

