/*
   Time frame measurement. Timer initialization. API.
   
   Changelog:
   	
   	2017-12-27: Initial release.

*/

#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/timer.h>
#include <libopencm3/cm3/nvic.h>

#include "led.h"

volatile uint_fast32_t ms = 0;

int_fast8_t timer_ms(uint_fast32_t m){
/*
	Returns true (1) if an interval of <m> milliseconds passed. In this case
	a new interval measurement is started. Returns false (0) otherwise.
	
	The first interval measurement is started after system reset. The current
	measurement may be restarted calling <timer_ms(0)>.
*/
	static uint_fast32_t start = 0;
	
	if(m <= ms - start){
		
		start = ms;
		return 1;
	}
	else{
		
		return 0;
	}
}

void timer_init(){

	
	rcc_periph_clock_enable(RCC_TIM6);
	nvic_enable_irq(NVIC_TIM6_DAC_IRQ);
	
	TIM6_DIER |= TIM_DIER_UIE;
	TIM6_ARR = 8000;
	TIM6_CR1 |= TIM_CR1_CEN;
}

void tim6_dac_isr(){
	
	if(TIM6_SR & TIM_SR_UIF){
	
		TIM6_SR &= ~TIM_SR_UIF;
		++ms;		
	}
}
